import * as Events from '@storybook/core-events';
import * as polyfill from 'event-source-polyfill';
import { buildQueries, within } from '@storybook/testing-library';
import { addons, makeDecorator } from '@storybook/addons';
import { isObject, noop } from '../../types';
import { denormalizeStoryParameters, serializeRawStories } from '../../shared';
import { getConnectionUrl } from '../shared/helpers';
import { isInternetExplorer } from './utils';

if (typeof process != 'object' || typeof process.version != 'string') {
  // NOTE If you don't use babel-polyfill or any other polyfills that add EventSource for IE11
  // You don't get hot reload in IE11. So put polyfill for that to better UX
  // Don't load in nodejs environment
  // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
  const {
    NativeEventSource,
    EventSourcePolyfill
  } = polyfill; // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

  window.EventSource = NativeEventSource || EventSourcePolyfill;
}

const disableAnimationsStyles = "\n*,\n*:hover,\n*::before,\n*::after {\n  animation-delay: -0.0001ms !important;\n  animation-duration: 0s !important;\n  animation-play-state: paused !important;\n  cursor: none !important;\n  caret-color: transparent !important;\n  transition: 0s !important;\n}\n";

async function resetCurrentStory(channel) {
  setTimeout(() => channel.emit(Events.SET_CURRENT_STORY, {
    storyId: true,
    name: '',
    kind: ''
  }), 0);
  return new Promise(resolve => channel.once(Events.STORY_MISSING, resolve));
}

function catchRenderError(channel) {
  let rejectCallback;
  const promise = new Promise((_resolve, reject) => rejectCallback = reject);

  function errorHandler(_ref) {
    let {
      title,
      description
    } = _ref;
    rejectCallback({
      message: title,
      stack: description
    });
  }

  function exceptionHandler(exception) {
    rejectCallback(exception);
  }

  function removeHandlers() {
    channel.off(Events.STORY_ERRORED, errorHandler);
    channel.off(Events.STORY_THREW_EXCEPTION, errorHandler);
  }

  channel.once(Events.STORY_ERRORED, errorHandler);
  channel.once(Events.STORY_THREW_EXCEPTION, exceptionHandler);
  return Object.assign(promise, {
    cancel: removeHandlers
  });
}

function waitForStoryRendered(channel) {
  let resolveCallback;
  const promise = new Promise(resolve => resolveCallback = resolve);

  function renderHandler() {
    resolveCallback();
  }

  function removeHandlers() {
    channel.off(Events.STORY_RENDERED, renderHandler);
  }

  channel.once(Events.STORY_RENDERED, renderHandler);
  return Object.assign(promise, {
    cancel: removeHandlers
  });
}

function waitForFontsLoaded() {
  if (!document.fonts) return;
  const areFontsLoading = Array.from(document.fonts).some(font => font.status == 'loading');

  if (areFontsLoading) {
    return new Promise(resolve => {
      const fontsLoadedHandler = () => {
        document.fonts.removeEventListener('loadingdone', fontsLoadedHandler);
        resolve();
      };

      document.fonts.addEventListener('loadingdone', fontsLoadedHandler);
    });
  }
}

function waitForCaptureCall() {
  return new Promise(resolve => captureResolver = resolve);
}

function initCreeveyState() {
  var _window$localStorage$;

  const prevState = JSON.parse((_window$localStorage$ = window.localStorage.getItem('Creevey_Tests')) !== null && _window$localStorage$ !== void 0 ? _window$localStorage$ : '{}');
  if (prevState.creeveyHost) window.__CREEVEY_SERVER_HOST__ = prevState.creeveyHost;
  if (prevState.creeveyPort) window.__CREEVEY_SERVER_PORT__ = prevState.creeveyPort;
  if (prevState.setStoriesCounter) setStoriesCounter = prevState.setStoriesCounter;
  if (prevState.isTestBrowser) isTestBrowser = prevState.isTestBrowser;
  window.addEventListener('beforeunload', () => {
    window.localStorage.setItem('Creevey_Tests', JSON.stringify({
      creeveyHost: window.__CREEVEY_SERVER_HOST__,
      creeveyPort: window.__CREEVEY_SERVER_PORT__,
      setStoriesCounter,
      isTestBrowser
    }));
  });
}

let isTestBrowser = false;
let captureResolver;
let waitForCreevey;
let creeveyReady;
let setStoriesCounter = 0;
export function withCreevey() {
  let currentStory = '';
  let isAnimationDisabled = false;
  initCreeveyState();

  function disableAnimation() {
    isAnimationDisabled = true;
    const style = document.createElement('style');
    const textNode = document.createTextNode(disableAnimationsStyles);
    style.setAttribute('type', 'text/css');
    style.appendChild(textNode);
    document.head.appendChild(style);
  }

  async function getStories() {
    var _window$__STORYBOOK_S;

    const storiesPromise = new Promise(resolve => addons.getChannel().once(Events.SET_STORIES, data => resolve(serializeRawStories(denormalizeStoryParameters(data)))));
    const store = (_window$__STORYBOOK_S = window.__STORYBOOK_STORY_STORE__) !== null && _window$__STORYBOOK_S !== void 0 ? _window$__STORYBOOK_S : {};

    if (store.cacheAllCSFFiles) {
      await store.cacheAllCSFFiles();
      addons.getChannel().emit(Events.SET_STORIES, store.getSetStoriesPayload());
    } else return;

    addons.getChannel().on(Events.SET_STORIES, data => {
      // TODO Figure out how to get only updated stories
      // TODO Subscribe on hmr? like use dummy-hmr
      setStoriesCounter += 1;
      const stories = serializeRawStories(denormalizeStoryParameters(data));
      const storiesByFiles = new Map();
      Object.values(stories).forEach(story => {
        const storiesFromFile = storiesByFiles.get(story.parameters.fileName);
        if (storiesFromFile) storiesFromFile.push(story);else storiesByFiles.set(story.parameters.fileName, [story]);
      });
      void fetch("http://".concat(getConnectionUrl(), "/stories"), {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: JSON.stringify({
          setStoriesCounter,
          stories: [...storiesByFiles.entries()]
        })
      });
    });
    return storiesPromise;
  }

  async function selectStory(storyId, kind, name, shouldWaitForReady, callback) {
    if (!isAnimationDisabled) disableAnimation();
    isTestBrowser = true;
    const channel = addons.getChannel();
    const waitForReady = shouldWaitForReady ? new Promise(resolve => window.__CREEVEY_SET_READY_FOR_CAPTURE__ = resolve) : Promise.resolve();
    if (storyId == currentStory) await resetCurrentStory(channel);else currentStory = storyId;
    let isCaptureCalled = false;
    const renderPromise = waitForStoryRendered(channel);
    const errorPromise = catchRenderError(channel);
    const capturePromise = waitForCaptureCall().then(() => isCaptureCalled = true);
    setTimeout(() => channel.emit(Events.SET_CURRENT_STORY, {
      storyId,
      name,
      kind
    }), 0);

    try {
      await Promise.race([(async () => {
        await Promise.race([renderPromise, capturePromise]);
        await waitForFontsLoaded();
        await waitForReady;
      })(), errorPromise]);
      callback([null, isCaptureCalled]);
    } catch (reason) {
      var _reason$stack;

      // NOTE Event `STORY_THREW_EXCEPTION` triggered only in react and vue frameworks and return Error instance
      // NOTE Event `STORY_ERRORED` return error-like object without `name` field
      const errorMessage = reason instanceof Error ? (_reason$stack = reason.stack) !== null && _reason$stack !== void 0 ? _reason$stack : reason.message : isObject(reason) ? "".concat(reason.message, "\n    ").concat(reason.stack) : reason;
      callback([errorMessage]);
    } finally {
      renderPromise.cancel();
      errorPromise.cancel();
    }
  }

  function updateGlobals(globals) {
    addons.getChannel().emit(Events.UPDATE_GLOBALS, {
      globals
    });
  }

  function insertIgnoreStyles(ignoreSelectors) {
    const stylesElement = document.createElement('style');
    stylesElement.setAttribute('type', 'text/css');
    document.head.appendChild(stylesElement);
    ignoreSelectors.forEach(selector => {
      stylesElement.innerHTML += "\n        ".concat(selector, " {\n          background: #000 !important;\n          box-shadow: none !important;\n          text-shadow: none !important;\n          outline: 0 !important;\n          color: rgba(0,0,0,0) !important;\n        }\n        ").concat(selector, " *, ").concat(selector, "::before, ").concat(selector, "::after {\n          visibility: hidden !important;\n        }\n      ");
    });
    return stylesElement;
  }

  function removeIgnoreStyles(ignoreStyles) {
    var _ignoreStyles$parentN;

    (_ignoreStyles$parentN = ignoreStyles.parentNode) === null || _ignoreStyles$parentN === void 0 ? void 0 : _ignoreStyles$parentN.removeChild(ignoreStyles);
  }

  function hasPlayCompletedYet(callback) {
    creeveyReady();
    let isCaptureCalled = false;
    let isPlayCompleted = false;
    const channel = addons.getChannel();
    void waitForStoryRendered(channel).then(() => {
      if (isCaptureCalled) return;
      isPlayCompleted = true;
      callback(true);
    });
    void waitForCaptureCall().then(() => {
      if (isPlayCompleted) return;
      isCaptureCalled = true;
      callback(false);
    });
  }

  window.__CREEVEY_GET_STORIES__ = getStories;
  window.__CREEVEY_SELECT_STORY__ = selectStory;
  window.__CREEVEY_UPDATE_GLOBALS__ = updateGlobals;
  window.__CREEVEY_INSERT_IGNORE_STYLES__ = insertIgnoreStyles;
  window.__CREEVEY_REMOVE_IGNORE_STYLES__ = removeIgnoreStyles;
  window.__CREEVEY_HAS_PLAY_COMPLETED_YET__ = hasPlayCompletedYet;
  window.__CREEVEY_SET_READY_FOR_CAPTURE__ = noop;

  const queryAllByQuery = (container, query) => [...container.querySelectorAll(query)].filter(e => e instanceof HTMLElement);

  const getMultipleError = (_, query) => "Found multiple elements by query: ".concat(query);

  const getMissingError = (_, query) => "Unable to find an element by query: ".concat(query);

  const [queryByQuery, getAllByQuery, getByQuery, findAllByQuery, findByQuery] = buildQueries(queryAllByQuery, getMultipleError, getMissingError);
  const queries = {
    queryByQuery,
    getAllByQuery,
    getByQuery,
    findAllByQuery,
    findByQuery
  };
  return makeDecorator({
    name: 'withCreevey',
    parameterName: 'creevey',
    wrapper: (getStory, context) => {
      var _ref2;

      // TODO Define proper types, like captureElement is a promise
      const {
        captureElement
      } = context.parameters.creevey = (_ref2 = context.parameters.creevey) !== null && _ref2 !== void 0 ? _ref2 : {};
      Object.defineProperty(context.parameters.creevey, 'captureElement', {
        get() {
          switch (true) {
            case captureElement === undefined:
              return Promise.resolve(context.canvasElement);

            case captureElement === null:
              return Promise.resolve(document.documentElement);

            case typeof captureElement == 'string':
              return isInternetExplorer // some code from testing-library makes IE hang
              ? Promise.resolve(context.canvasElement.querySelector(captureElement)) : within(context.canvasElement, queries).findByQuery(captureElement);

            case typeof captureElement == 'function':
              // TODO Define type for it
              return Promise.resolve(captureElement(context));
          }
        },

        enumerable: true,
        configurable: true
      });
      return getStory(context);
    }
  });
}
export async function capture(options) {
  if (!isTestBrowser) return;
  captureResolver();
  waitForCreevey = new Promise(resolve => creeveyReady = resolve);
  await fetch("http://".concat(getConnectionUrl(), "/capture"), {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      workerId: window.__CREEVEY_WORKER_ID__,
      options
    })
  });
  await waitForCreevey;
}