import { rmdirSync, writeFile } from 'fs';
import path from 'path';
import webpack from 'webpack';
import nodeExternals from 'webpack-node-externals';
import { extensions as fallbackExtensions, getCreeveyCache } from '../../utils';
import { getStorybookFramework, hasDocsAddon, importStorybookConfig, resolveFromStorybook } from '../../storybook/helpers';
import { noop } from '../../../types';
import { emitWebpackMessage, subscribeOn } from '../../messages';
import { logger } from '../../logger';
let isInitiated = false;
let dumpStats = noop;

function handleWebpackBuild(error, stats) {
  var _stats$toJson$warning;

  dumpStats(stats);

  if (error || !stats || stats.hasErrors()) {
    emitWebpackMessage({
      type: isInitiated ? 'rebuild failed' : 'fail'
    });
    console.error('=> Failed to build the Storybook preview bundle');
    if (error) return console.error(error.message);

    if (stats && (stats.hasErrors() || stats.hasWarnings())) {
      var _statsJson$errors, _statsJson$warnings;

      const statsJson = stats.toJson();
      (_statsJson$errors = statsJson.errors) === null || _statsJson$errors === void 0 ? void 0 : _statsJson$errors.forEach(e => console.error(e));
      (_statsJson$warnings = statsJson.warnings) === null || _statsJson$warnings === void 0 ? void 0 : _statsJson$warnings.forEach(e => console.error(e));
      return;
    }
  }

  stats === null || stats === void 0 ? void 0 : (_stats$toJson$warning = stats.toJson().warnings) === null || _stats$toJson$warning === void 0 ? void 0 : _stats$toJson$warning.forEach(e => console.warn(e));

  if (!isInitiated) {
    isInitiated = true;
    emitWebpackMessage({
      type: 'success'
    });
  } else {
    emitWebpackMessage({
      type: 'rebuild succeeded'
    });
  }

  return;
}

async function applyMdxLoader(config, areAddonsRemoved, loader) {
  const {
    mdxLoaders
  } = await import('./mdx-loader');
  mdxLoaders.splice(1, 0, loader);
  const mdxRegexps = [/(stories|story)\.mdx$/]; // NOTE replace md/mdx to null loader

  const mdRegexps = [/\.md$/, /\.mdx$/];

  if (areAddonsRemoved) {
    var _config$module2, _config$module2$rules;

    mdRegexps.forEach(test => {
      var _config$module, _config$module$rules;

      return (_config$module = config.module) === null || _config$module === void 0 ? void 0 : (_config$module$rules = _config$module.rules) === null || _config$module$rules === void 0 ? void 0 : _config$module$rules.unshift({
        test,
        exclude: /(stories|story)\.mdx$/,
        use: require.resolve('null-loader')
      });
    });
    (_config$module2 = config.module) === null || _config$module2 === void 0 ? void 0 : (_config$module2$rules = _config$module2.rules) === null || _config$module2$rules === void 0 ? void 0 : _config$module2$rules.unshift({
      test: /(stories|story)\.mdx$/,
      use: mdxLoaders
    });
  } else {
    var _config$module3, _config$module3$rules, _config$module4, _config$module4$rules, _config$module5, _config$module5$rules;

    // NOTE Exclude addons' entry points
    config.entry = Array.isArray(config.entry) ? config.entry.filter(entry => !/@storybook(\/|\\)addon/.test(entry)) : config.entry;
    (_config$module3 = config.module) === null || _config$module3 === void 0 ? void 0 : (_config$module3$rules = _config$module3.rules) === null || _config$module3$rules === void 0 ? void 0 : _config$module3$rules.flatMap(rule => typeof rule == 'object' && 'test' in rule ? rule : []).filter(rule => mdRegexps.some(test => {
      var _rule$test;

      return ((_rule$test = rule.test) === null || _rule$test === void 0 ? void 0 : _rule$test.toString()) == test.toString();
    })).forEach(rule => rule.use = require.resolve('null-loader'));
    (_config$module4 = config.module) === null || _config$module4 === void 0 ? void 0 : (_config$module4$rules = _config$module4.rules) === null || _config$module4$rules === void 0 ? void 0 : _config$module4$rules.flatMap(rule => typeof rule == 'object' && 'test' in rule ? rule : []).filter(rule => mdxRegexps.some(test => {
      var _rule$test2;

      return ((_rule$test2 = rule.test) === null || _rule$test2 === void 0 ? void 0 : _rule$test2.toString()) == test.toString();
    })).forEach(rule => rule.use = mdxLoaders); // NOTE Exclude source-loader

    config.module = { ...config.module,
      rules: ((_config$module5 = config.module) === null || _config$module5 === void 0 ? void 0 : (_config$module5$rules = _config$module5.rules) === null || _config$module5$rules === void 0 ? void 0 : _config$module5$rules.flatMap(rule => typeof rule == 'object' && 'test' in rule ? rule : []).filter(rule => !(typeof rule.loader == 'string' && /@storybook(\/|\\)source-loader/.test(rule.loader)))) ?? []
    };
  }
}

async function getWebpackConfigForStorybook_6_2(framework, configDir, outputDir) {
  const {
    default: storybookFrameworkOptions
  } = await import(resolveFromStorybook(`@storybook/${framework}/dist/cjs/server/options`));
  const options = {
    quiet: true,
    configType: 'PRODUCTION',
    outputDir,
    configDir,
    ...storybookFrameworkOptions
  }; // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

  const {
    getPreviewBuilder
  } = await import(resolveFromStorybook('@storybook/core-server/dist/cjs/utils/get-preview-builder')); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

  const {
    loadAllPresets
  } = await import(resolveFromStorybook('@storybook/core-common')); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

  const builder = await getPreviewBuilder(configDir); // NOTE: Copy-paste from storybook/lib/core-server/src/build-dev.ts
  // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

  const presets = loadAllPresets({
    corePresets: [// eslint-disable-next-line node/no-missing-require
    resolveFromStorybook('@storybook/core-server/dist/cjs/presets/common-preset'), // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
    ...builder.corePresets, // eslint-disable-next-line node/no-missing-require
    resolveFromStorybook('@storybook/core-server/dist/cjs/presets/babel-cache-preset')],
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
    overridePresets: [...(hasDocsAddon() ? [require.resolve('./mdx-loader')] : []), // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
    ...builder.overridePresets],
    ...options
  }); // eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call, @typescript-eslint/no-unsafe-assignment

  return builder.getConfig({ ...options,
    presets
  });
}

async function removeAddons() {
  try {
    var _config$core;

    const config = await importStorybookConfig();

    if (((_config$core = config.core) === null || _config$core === void 0 ? void 0 : _config$core.builder) == 'webpack5') {
      logger.warn("Be aware Creevey doesn't fully support webpack@5, some feature might not work well");
    }

    if (config.addons && config.stories) {
      config.addons = [];
      return true;
    }
  } catch (_) {
    /* noop */
  }

  return false;
}

export default async function compile(config, {
  debug,
  ui
}) {
  var _storybookWebpackConf, _storybookWebpackConf2, _storybookWebpackConf3, _extensions$map, _storybookWebpackConf4, _storybookWebpackConf5;

  const storybookFramework = getStorybookFramework();
  const outputDir = path.join(getCreeveyCache(), 'storybook');

  try {
    rmdirSync(outputDir, {
      recursive: true
    });
  } catch (_) {
    /* noop */
  }

  const creeveyLoader = {
    loader: require.resolve('./creevey-loader'),
    options: {
      debug,
      storybookDir: config.storybookDir
    }
  };
  process.env.NODE_ENV = 'production'; // NOTE Remove addons by monkey patching, only for new config file (main.js)

  const areAddonsRemoved = await removeAddons();
  const getWebpackConfig = getWebpackConfigForStorybook_6_2;
  const storybookWebpackConfig = await getWebpackConfig(storybookFramework, config.storybookDir, outputDir);
  const extensions = ((_storybookWebpackConf = storybookWebpackConfig.resolve) === null || _storybookWebpackConf === void 0 ? void 0 : _storybookWebpackConf.extensions) ?? fallbackExtensions;
  delete storybookWebpackConfig.optimization;
  storybookWebpackConfig.devtool = false;
  storybookWebpackConfig.performance = false;
  storybookWebpackConfig.profile = debug;
  storybookWebpackConfig.mode = 'development';
  storybookWebpackConfig.target = 'node';
  storybookWebpackConfig.output = { ...storybookWebpackConfig.output,
    filename: 'main.js'
  }; // NOTE Add hack to allow stories HMR work in nodejs

  Array.isArray(storybookWebpackConfig.entry) && storybookWebpackConfig.entry.unshift(require.resolve('./dummy-hmr')); // NOTE apply creevey loader to output from mdx loader

  if (hasDocsAddon()) await applyMdxLoader(storybookWebpackConfig, areAddonsRemoved, creeveyLoader); // NOTE Add creevey-loader to cut off all unnecessary code except stories meta and tests

  (_storybookWebpackConf2 = storybookWebpackConfig.module) === null || _storybookWebpackConf2 === void 0 ? void 0 : (_storybookWebpackConf3 = _storybookWebpackConf2.rules) === null || _storybookWebpackConf3 === void 0 ? void 0 : _storybookWebpackConf3.unshift({
    enforce: 'pre',
    test: new RegExp(`\\.(${(_extensions$map = extensions.map(x => x.slice(1))) === null || _extensions$map === void 0 ? void 0 : _extensions$map.join('|')})$`),
    exclude: /node_modules/,
    use: creeveyLoader
  });
  const aliases = ((_storybookWebpackConf4 = storybookWebpackConfig.resolve) === null || _storybookWebpackConf4 === void 0 ? void 0 : _storybookWebpackConf4.alias) ?? {};
  const excluded = ['@storybook/addons', '@storybook/api', '@storybook/channel-postmessage', '@storybook/channels', '@storybook/client-api', '@storybook/client-logger', '@storybook/components', '@storybook/core-events', '@storybook/router', '@storybook/semver', '@storybook/theming']; // NOTE Exclude from bundle all modules from node_modules

  storybookWebpackConfig.externals = [...(Array.isArray(aliases) ? aliases.map(({
    name,
    alias
  }) => [name, alias]) : Object.entries(aliases)).filter(([alias]) => excluded.includes(alias)).flatMap(([, aliasPath]) => aliasPath == false ? [] : (Array.isArray(aliasPath) ? aliasPath : [aliasPath]).map(x => ({
    [x]: `commonjs ${x}`
  }))), // NOTE Replace `@storybook/${framework}` to ../../storybook.ts
  {
    [`@storybook/${storybookFramework}`]: `commonjs ${require.resolve('../../storybook/entry')}`
  }, nodeExternals({
    includeAbsolutePaths: true,
    allowlist: /(webpack|dummy-hmr|generated-stories-entry|generated-config-entry|generated-other-entry)/
  }), // TODO Don't work well with monorepos
  nodeExternals({
    modulesDir: resolveFromStorybook('@storybook/core-client').split('@storybook')[0],
    includeAbsolutePaths: true,
    allowlist: /(webpack|dummy-hmr|generated-stories-entry|generated-config-entry|generated-other-entry)/
  })]; // NOTE Exclude some plugins

  const excludedPlugins = ['DocgenPlugin', 'ForkTsCheckerWebpackPlugin'];
  storybookWebpackConfig.plugins = (_storybookWebpackConf5 = storybookWebpackConfig.plugins) === null || _storybookWebpackConf5 === void 0 ? void 0 : _storybookWebpackConf5.filter(plugin => !excludedPlugins.includes(plugin.constructor.name));
  const storybookWebpackCompiler = webpack(storybookWebpackConfig);

  if (debug) {
    dumpStats = stats => {
      if (!stats) return;
      writeFile(path.join(config.reportDir, 'stats.json'), JSON.stringify(stats.toJson(), null, 2), noop);
    };
  }

  if (ui) {
    const watcher = storybookWebpackCompiler.watch({}, handleWebpackBuild);
    subscribeOn('shutdown', () => watcher.close(noop));
  } else {
    storybookWebpackCompiler.run(handleWebpackBuild);
  }
}