import path from 'path';
import resolveFrom from 'resolve-from';
const supportedFrameworks = ['react', 'vue', 'vue3', 'angular', 'marionette', 'mithril', 'marko', 'html', 'svelte', 'riot', 'ember', 'preact', 'rax', 'aurelia', 'server', 'web-components'];
export const storybookDirRef = {
  current: path.resolve('.storybook')
};
export const resolveFromStorybook = modulePath => resolveFrom(storybookDirRef.current, modulePath);
export const resolveFromStorybookAddonDocs = modulePath => resolveFrom(resolveFromStorybook('@storybook/addon-docs'), modulePath);
export const resolveFromStorybookBuilderWebpack4 = modulePath => resolveFrom(resolveFromStorybook('@storybook/builder-webpack4'), modulePath);
export const resolveFromStorybookCoreClient = modulePath => resolveFrom(resolveFromStorybook('@storybook/core-client'), modulePath);
export const resolveFromStorybookCoreServer = modulePath => resolveFrom(resolveFromStorybook('@storybook/core-server'), modulePath);

const importFromStorybook = modulePath => import(resolveFromStorybook(modulePath));

export const importStorybookClientLogger = () => importFromStorybook('@storybook/client-logger');
export const importStorybookCoreCommon = () => importFromStorybook('@storybook/core-common');
export const importStorybookCoreEvents = () => importFromStorybook('@storybook/core-events');
export function hasDocsAddon() {
  try {
    resolveFromStorybook('@storybook/addon-docs');
    return true;
  } catch (_) {
    return false;
  }
}
export function hasSvelteCSFAddon() {
  try {
    resolveFromStorybook('@storybook/addon-svelte-csf');
    return true;
  } catch (_) {
    return false;
  }
}
export function getStorybookVersion() {
  // eslint-disable-next-line @typescript-eslint/no-var-requires
  const {
    version
  } = require(resolveFromStorybook('@storybook/core/package.json'));

  return version;
}
export function isStorybookVersionLessThan(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) < major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) < minor;
}
export function isStorybookVersionGreaterThan(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) > major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) > minor;
}
export function isStorybookVersion(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) == major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) == minor;
}
export function getStorybookFramework() {
  const framework = process.env.__CREEVEY_STORYBOOK_FRAMEWORK__ ?? supportedFrameworks.find(framework => {
    try {
      return require.resolve(resolveFromStorybook(`@storybook/${framework}`));
    } catch (_) {
      return false;
    }
  });
  if (!framework) throw new Error("Couldn't detect used Storybook framework. Please ensure that you install `@storybook/<framework>` package");
  return framework;
}
export const storybookConfigRef = {
  current: {
    stories: []
  }
};
export async function importStorybookConfig() {
  const configPath = `${storybookDirRef.current}/main`;

  try {
    return storybookConfigRef.current = (await import(require.resolve(configPath))).default;
  } catch (_) {
    const storybookUtilsPath = '@storybook/core-common/dist/cjs/utils';
    const serverRequireModule = 'interpret-require'; // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

    const {
      getInterpretedFile
    } = await import(resolveFromStorybook(`${storybookUtilsPath}/interpret-files`)); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

    const {
      default: serverRequireFallback,
      serverRequire = serverRequireFallback
    } = await import(resolveFromStorybook(`${storybookUtilsPath}/${serverRequireModule}`)); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

    const mainConfigFile = getInterpretedFile(configPath); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

    return storybookConfigRef.current = serverRequire(mainConfigFile);
  }
}
export async function isCSFv3Enabled() {
  var _await$importStoryboo, _await$importStoryboo2;

  return ((_await$importStoryboo = await importStorybookConfig()) === null || _await$importStoryboo === void 0 ? void 0 : (_await$importStoryboo2 = _await$importStoryboo.features) === null || _await$importStoryboo2 === void 0 ? void 0 : _await$importStoryboo2.previewCsfV3) ?? false;
}