/* eslint-disable @typescript-eslint/no-var-requires, @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access,@typescript-eslint/ban-ts-comment */
import path from 'path';
import cluster from 'cluster';
import chokidar from 'chokidar';
import { noop } from '../../../types';
import { getCreeveyCache } from '../../utils';
import { subscribeOn } from '../../messages';
import { importStorybookClientLogger, importStorybookConfig, importStorybookCoreCommon, importStorybookCoreEvents } from '../helpers';
import { logger } from '../../logger';

async function initStorybookEnvironment() {
  // @ts-expect-error There is no @types/global-jsdom package
  (await import('global-jsdom')).default(undefined, {
    url: 'http://localhost'
  }); // NOTE Cutoff `jsdom` part from userAgent, because storybook check enviroment and create events channel if runs in browser
  // https://github.com/storybookjs/storybook/blob/v5.2.8/lib/core/src/client/preview/start.js#L98
  // Example: "Mozilla/5.0 (linux) AppleWebKit/537.36 (KHTML, like Gecko) jsdom/15.2.1"

  Object.defineProperty(window.navigator, 'userAgent', {
    value: window.navigator.userAgent.split(' ').filter(token => !token.startsWith('jsdom')).join(' ')
  }); // TODO Look at creevey debug flag

  const {
    logger
  } = await importStorybookClientLogger(); // NOTE: Disable duplication warnings for >=6.2 storybook

  if (cluster.isWorker) logger.warn = noop; // NOTE: disable logger for 5.x storybook

  logger.debug = noop;
  return import('../entry');
}

function watchStories(channel, watcher, initialFiles) {
  const watchingFiles = initialFiles;
  let storiesByFiles = new Map();
  subscribeOn('shutdown', () => void watcher.close());
  watcher.add(Array.from(watchingFiles));
  watcher.on('change', filePath => storiesByFiles.set(path.isAbsolute(filePath) ? filePath : `./${filePath.replace(/\\/g, '/')}`, []));
  watcher.on('unlink', filePath => storiesByFiles.set(path.isAbsolute(filePath) ? filePath : `./${filePath.replace(/\\/g, '/')}`, []));
  return data => {
    const stories = data.stories;
    const files = new Set(Object.values(stories).map(story => story.parameters.fileName));
    const addedFiles = Array.from(files).filter(filePath => !watchingFiles.has(filePath));
    const removedFiles = Array.from(watchingFiles).filter(filePath => !files.has(filePath));
    watcher.add(addedFiles);
    addedFiles.forEach(filePath => {
      watchingFiles.add(filePath);
      storiesByFiles.set(filePath, []);
    });
    removedFiles.forEach(filePath => watchingFiles.delete(filePath));
    Object.values(stories).forEach(story => {
      var _storiesByFiles$get;

      return (_storiesByFiles$get = storiesByFiles.get(story.parameters.fileName)) === null || _storiesByFiles$get === void 0 ? void 0 : _storiesByFiles$get.push(story);
    });
    channel.emit('storiesUpdated', storiesByFiles);
    storiesByFiles = new Map();
  };
}

function loadStoriesFromBundle(watch) {
  const bundlePath = path.join(getCreeveyCache(), 'storybook/main.js');

  if (watch) {
    subscribeOn('webpack', message => {
      if (message.type != 'rebuild succeeded') return;
      Object.values(global.__CREEVEY_HMR_DATA__).filter(({
        callback
      }) => callback).forEach(({
        data,
        callback
      }) => callback(data));
      delete require.cache[bundlePath];
      import(bundlePath);
    });
  }

  import(bundlePath);
}

async function loadStoriesDirectly(config, {
  watcher,
  debug
}) {
  const {
    toRequireContext,
    normalizeStoriesEntry
  } = await importStorybookCoreCommon();
  const {
    addParameters,
    configure
  } = await import('../entry');
  const requireContext = await (await import('../../loaders/babel/register')).default(config, debug);

  const preview = (() => {
    try {
      return require.resolve(`${config.storybookDir}/preview`);
    } catch (_) {
      /* noop */
    }
  })();

  const {
    stories
  } = await importStorybookConfig();
  const contexts = stories.map(entry => {
    const normalizedEntry = normalizeStoriesEntry(entry, {
      configDir: config.storybookDir,
      workingDir: process.cwd()
    });
    const {
      path: storiesPath,
      recursive,
      match
    } = toRequireContext(normalizedEntry);
    watcher === null || watcher === void 0 ? void 0 : watcher.add(path.resolve(config.storybookDir, storiesPath));
    return () => requireContext(storiesPath, recursive, new RegExp(match));
  });

  let disposeCallback = data => void data;

  Object.assign(module, {
    hot: {
      data: {},

      accept() {
        /* noop */
      },

      dispose(callback) {
        disposeCallback = callback;
      }

    }
  });

  async function startStorybook() {
    if (preview) {
      const {
        parameters,
        globals,
        globalTypes
      } = await import(preview);
      if (parameters) addParameters(parameters);
      if (globals) addParameters({
        globals
      });
      if (globalTypes) addParameters({
        globalTypes
      });
    }

    try {
      configure(contexts.map(ctx => ctx()), module, false);
    } catch (error) {
      if (cluster.isPrimary) logger.error(error);
    }
  }

  watcher === null || watcher === void 0 ? void 0 : watcher.add(config.storybookDir);
  watcher === null || watcher === void 0 ? void 0 : watcher.on('all', (_event, filename) => {
    var _module$hot;

    disposeCallback((_module$hot = module.hot) === null || _module$hot === void 0 ? void 0 : _module$hot.data);
    delete require.cache[filename];
    void startStorybook();
  });
  void startStorybook();
} // TODO Do we need to support multiple storybooks here?


export const loadStories = async (config, {
  watch,
  debug
}, storiesListener) => {
  const storybookApi = await initStorybookEnvironment();
  const Events = await importStorybookCoreEvents();
  const {
    channel
  } = storybookApi;
  channel.removeAllListeners(Events.CURRENT_STORY_WAS_SET);
  channel.on('storiesUpdated', storiesListener);
  let watcher;
  if (watch) watcher = chokidar.watch([], {
    ignoreInitial: true
  });
  const loadPromise = new Promise(resolve => {
    channel.once(Events.SET_STORIES, data => {
      const stories = data.stories;
      const files = new Set(Object.values(stories).map(story => story.parameters.fileName));
      if (watcher) channel.on(Events.SET_STORIES, watchStories(channel, watcher, files));
      resolve(stories);
    });
  });
  if (config.useWebpackToExtractTests) loadStoriesFromBundle(watch);else void loadStoriesDirectly(config, {
    watcher,
    debug
  });
  return loadPromise;
};
export async function extractStoriesData(config, {
  watch,
  debug
}) {
  const storybookApi = await initStorybookEnvironment();
  const Events = await importStorybookCoreEvents();
  const {
    channel
  } = storybookApi;
  channel.removeAllListeners(Events.CURRENT_STORY_WAS_SET);
  const loadPromise = new Promise(resolve => channel.once(Events.SET_STORIES, resolve));
  if (config.useWebpackToExtractTests) loadStoriesFromBundle(watch);else void loadStoriesDirectly(config, {
    debug
  });
  return loadPromise;
}